<?php

/**
 * Riblink.com API PHP Client
 * @link https://riblink.com/dashboard/tools#tools_api
 * @link https://api.riblink.com
 */

class RibLinkApi {
	
	/**
	 * Define your user ID
	 * @var int 
	 */
	protected static $_userId = 0;
	
	/**
	 * Define your API key
	 * @var string 
	 */
	protected static $_userApiKey = '';
	
	/**
	 * Define your language code (ISO 639-1). 
	 * See API documentation for supported languages
	 * @var string 
	 */
	protected static $_apiLang = 'en';
	
	/**
	 * Define response data type (json, xml)
	 * See API documentation for supported data types
	 * @var string 
	 */
	protected static $_responseDataType = 'json';
	
	/**
	 * API request URL
	 * @var string 
	 */
	protected static $_apiUrl = 'https://api.riblink.com';
	
	/**
	 * Client ID. Don't change this line for compatibility support
	 * @var string 
	 */
	protected static $_clientId = 'Riblink.com API PHP Client v1.2';

	
	/**
	 * Initialization
	 * 
	 * You can use this method or define all constants direct in the class
	 * 
	 * @param array $params Initialization parameters 
	 * ['user_id' => your User ID, 'api_key' => your API key, 'lang' => language code]
	 */
	public static function init(array $params) {
		static::$_userId = $params['user_id'];
		static::$_userApiKey = $params['api_key'];
		
		if (isset($params['lang'])) {
			static::$_apiLang = $params['lang'];
		}
		
		if (isset($params['data_type'])) {
			static::$_responseDataType = $params['data_type'];
		}
		
		if (isset($params['api_url'])) {
			static::$_apiUrl = $params['api_url'];
		}
	}
	
	/**
	 * Performs request to the API
	 * 
	 * @param string $url Contains operational URL (see API documentation)
	 * @param array $postParams (optional) Data passed as API request (see API documentation)
	 * @param bool $returnObject (optional) Returns \stdClass object if $returnObject set TRUE. Default: TRUE
	 * 
	 * @return string|object Returns plain response string or \stdClass object
	 */
	public static function request($url, array $postParams = [], $returnObject = true) {
		if (!static::$_userId || !static::$_userApiKey)
			return static::_error('User ID or API key not defined');
		
		$url = trim(static::$_apiUrl, '/') 
				. '/' . static::$_apiLang 
				. '/request/' 
				. static::$_responseDataType 
				. '/' . trim($url, '/');
		
		$postParams = array_merge([
			'user_id' => static::$_userId,
		], $postParams);
		
		ksort($postParams);
		
		$securityKey = hash_hmac('sha256', implode(':', $postParams), static::$_userApiKey, false);
		
		// URLs must be encoded so encode all parameters by default
		foreach ($postParams as &$param)
			$param = rawurlencode($param);
		unset($param);
		
		$postQuery = http_build_query($postParams) . '&skey=' . $securityKey;

		$curlOptions = [
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_HEADER => false,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_MAXREDIRS => 3,
            CURLOPT_ENCODING => 'utf-8',
            CURLOPT_USERAGENT => static::$_clientId,
            CURLOPT_AUTOREFERER => true,
            CURLOPT_FAILONERROR => true,
            CURLOPT_CONNECTTIMEOUT => 30,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_SSL_VERIFYHOST => 0,
			CURLOPT_POST => true,
			CURLOPT_POSTFIELDS => $postQuery
        ];
		
        $ch = curl_init($url);
        
		curl_setopt_array($ch, $curlOptions);
		
        $response = curl_exec($ch);
		$error = curl_error($ch);
		
        curl_close($ch);
		
        if ($error) {
			return static::_error($error);
		}
		
		if (!$returnObject) {
			return $response;
		} else {
			if (static::$_responseDataType == 'xml') {
				return simplexml_load_string($response);
			} else {
				return json_decode($response);
			}
		}
	}
	
	/**
	 * Request error method
	 * 
	 * Throws catchable \Exception if request can't be performed
	 * 
	 * @param string $message Error message
	 * @throws \Exception
	 */
	protected static function _error($message) {
		throw new \Exception(__CLASS__ . ' ERROR: ' . $message);
	}
}